@@ -36,6 +36,7 @@ from monotonic import monotonic as now
 import six
 
 from tenacity import _utils
+from tenacity import wait as _wait
 
 # Import all built-in retry strategies for easier usage.
 from .retry import retry_all  # noqa
@@ -93,7 +94,8 @@ def retry(*dargs, **dkw):
         def wrap(f):
             if asyncio and asyncio.iscoroutinefunction(f):
                 r = AsyncRetrying(*dargs, **dkw)
-            elif tornado and tornado.gen.is_coroutine_function(f):
+            elif tornado and hasattr(tornado.gen, 'is_coroutine_function') \
+                    and tornado.gen.is_coroutine_function(f):
                 r = TornadoRetrying(*dargs, **dkw)
             else:
                 r = Retrying(*dargs, **dkw)
@@ -121,6 +123,21 @@ class DoSleep(float):
 _unset = object()
 
 
+class RetryError(Exception):
+    """Encapsulates the last attempt instance right before giving up."""
+
+    def __init__(self, last_attempt):
+        self.last_attempt = last_attempt
+
+    def reraise(self):
+        if self.last_attempt.failed:
+            raise self.last_attempt.result()
+        raise self
+
+    def __str__(self):
+        return "{0}[{1}]".format(self.__class__.__name__, self.last_attempt)
+
+
 class BaseRetrying(object):
 
     def __init__(self,
@@ -129,7 +146,8 @@ class BaseRetrying(object):
                  retry=retry_if_exception_type(),
                  before=before_nothing,
                  after=after_nothing,
-                 reraise=False):
+                 reraise=False,
+                 retry_error_cls=RetryError):
         self.sleep = sleep
         self.stop = stop
         self.wait = wait
@@ -138,6 +156,11 @@ class BaseRetrying(object):
         self.after = after
         self.reraise = reraise
         self._local = threading.local()
+        # This will allow for passing in the result and handling
+        # the older versions of these functions that do not take
+        # the prior result.
+        self._wait_takes_result = self._waiter_takes_last_result(wait)
+        self.retry_error_cls = retry_error_cls
 
     def copy(self, sleep=_unset, stop=_unset, wait=_unset,
              retry=_unset, before=_unset, after=_unset, reraise=_unset):
@@ -152,6 +175,15 @@ class BaseRetrying(object):
             reraise=self.reraise if after is _unset else reraise,
         )
 
+    @staticmethod
+    def _waiter_takes_last_result(waiter):
+        if not six.callable(waiter):
+            return False
+        if isinstance(waiter, _wait.wait_base):
+            waiter = waiter.__call__
+        waiter_spec = _utils.getargspec(waiter)
+        return 'last_result' in waiter_spec.args
+
     def __repr__(self):
         attrs = dict(
             _utils.visible_attrs(self, attrs={'me': id(self)}),
@@ -246,13 +278,18 @@ class BaseRetrying(object):
             delay_since_first_attempt
         if self.stop(self.statistics['attempt_number'],
                      delay_since_first_attempt):
+            retry_exc = self.retry_error_cls(fut)
             if self.reraise:
-                raise RetryError(fut).reraise()
-            six.raise_from(RetryError(fut), fut.exception())
+                raise retry_exc.reraise()
+            six.raise_from(retry_exc, fut.exception())
 
         if self.wait:
-            sleep = self.wait(self.statistics['attempt_number'],
-                              delay_since_first_attempt)
+            if self._wait_takes_result:
+                sleep = self.wait(self.statistics['attempt_number'],
+                                  delay_since_first_attempt, last_result=fut)
+            else:
+                sleep = self.wait(self.statistics['attempt_number'],
+                                  delay_since_first_attempt)
         else:
             sleep = 0
         self.statistics['idle_for'] += sleep
@@ -314,21 +351,6 @@ class Future(futures.Future):
         return fut
 
 
-class RetryError(Exception):
-    """Encapsulates the last attempt instance right before giving up."""
-
-    def __init__(self, last_attempt):
-        self.last_attempt = last_attempt
-
-    def reraise(self):
-        if self.last_attempt.failed:
-            raise self.last_attempt.result()
-        raise self
-
-    def __str__(self):
-        return "RetryError[{0}]".format(self.last_attempt)
-
-
 if asyncio:
     from tenacity.async import AsyncRetrying
 
